// Fichier main.c
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *          Code source du programme principal			*
 * 														*
 *                                                      * 
 * .Date de cration : 19/01/09                         *
 * .Auteur : Maxime Pasquier	                        *       
 *                                                      *
 * .Descriptions :                                      *
 *      - code des fonctions pour la carte principale   *
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *                                                      *
 ********************************************************/
  
// Configuration du PIC
#pragma config WDT=OFF
#pragma config OSC=HS
#pragma config LVP=OFF

// Library du PIC 18f452
#include <p18f452.h>
#include <delays.h>
#include <timers.h>
#include <adc.h>
#include "CAN.h"

// Fonction dans d'autre fichier
#include "IO.h"
#include "moteur.h"
#include "I2C_Master.h"
#include "robot.h"

//	Configuration du timer 3
#define 	T_TIMER3 	40536			   	// 10 ms avant dbordement du Timer
											// Priode du quartz : 1/20e6 = 50ns
											// Priode  l'entre du prescaler : 50ns.4 = 200ns
											// Priode  la sortie du prescaler : 200ns.1 = 200ns
											// Nombre de priodes avant dbordement du timer : 10000 incrments
											// Priode entre deux interruptions : 5000*200ns = 1ms
											// Priode de 1ms -> 1KHz

											// 60536 pour 1ms
											// 55536 pour 2ms
											// ...
											// 40536 pour 5ms
											// ...
											// 15535 pour 10ms

// Adresse des cartes capteurs
#define AdCPT_M 0x40
#define AdCPT_G 0x4F
#define AdCPT_D 0x46

#define INC_FIG 8800	// 8500
#define V_FIG 175		// 230

void Interrupt(void);		// Utilise pour le timer
void figure ( void );

volatile int kv=KV;

/***************************************************************************************/
/*******************************************************/
char fig=0;
char prio=0;
char racourci=0;
/****************************/
void main ( void ){
	char i,j,k,l,m;
	unsigned char V=0x55;
	int VG = 0, VD = 0;
	int ressive = 0x00;
	unsigned char DATA[2]={0x00};
	int distanceG = 0;
	int distanceD = 0;
	int distance=0;

	ConfigProcessor();		// Configuration des entres et sorties
	CAN_Init();
	Moteur_Init();			// Initialisation de commandes des moteurs
	I2C_Init();				// Initialisation du port IC

//---------------------------------------------------------------------------------
			OpenTimer3(	TIMER_INT_ON &			// Autorise les interruption sur le timer 3
					T3_16BIT_RW &				// Compteur 16 bits
					T3_SOURCE_INT &				// Source interne (Quartz Fosc/4)
					T3_PS_1_1 &					// Prescaler 1/1
					T3_SYNC_EXT_OFF);			// Pas de synchronisation externe
		
			WriteTimer3(T_TIMER3);				// Charge le compteur du timer 3 avec la valeur de T_Timer3
			PIR2bits.TMR3IF=0;					// Timer3 Interrupt Flag = 0 Baisse le flag du timer 3
			IPR2bits.TMR3IP=0;					// Timer 3 dclench sur priorit basse
		
			RCONbits.IPEN=1;					// Autorise les niveau de priorit sur les interuptions
			kv = 0;
			// Autorise les interruptions	
/**/			INTCONbits.PEIE=0;					// Autorise les interruptions de basse priorit
/**/			INTCONbits.GIE=0;					// Autorise les interruptions de haute priorit
//---------------------------------------------------------------------------------			
	///////////////////////////////////////
/*///////////////////////////////////////////*/
/*/			  Boucle principale				/*/
/*///////////////////////////////////////////*/
	Moteur_Commande(0 ,1 );		// On stop les moteurs, bin oui c'est le dbut
	Moteur_Commande(0 ,2 );
	while ( !JACK ) {			// On attend que le jack soit rentrer
		LED=!LED;				// La la LED clignote
		Delay10KTCYx(200);
	}
	INTCONbits.PEIE=1;			// Quand le jack est dedans				
	INTCONbits.GIE=1;			// On lance la fonction des moteurs (voir routine d'intruption)
	kv=0;						// On lui dit de ce centrer sur la ligne 

	while ( JACK ) {			// On attend qu'on enlve le jack			
		LED=!LED;				// La LED clignote plus vite qu'avant (50<200)
		Delay10KTCYx(50);
	}


	kv=KV;						// Quand ya plus de jack en voiture simone
//////////////////////////////////////////
		LED = 1;
//////////////////////////////////////////
	while ( 1 )				// Tant qu'on a pas les clics de fin...
	{
		/*if ( prio ) {
			prio = 0;
			INTCONbits.PEIE=0;				
			INTCONbits.GIE=0;
			Moteur_Commande(0 ,1 );		// On stop les moteurs, bin oui c'est le dbut
			Moteur_Commande(0 ,2 );
			Delay10KTCYx(500);
			Delay10KTCYx(500);
			INTCONbits.PEIE=1;				
			INTCONbits.GIE=1;
		}*/
	//	if ( fig ) {			// Si on a dtecter une figures...
		//	INTCONbits.PEIE=0;	// On la fait, logique				
		//	INTCONbits.GIE=0;	
			
	/*		Moteur_Commande(200 ,1 );
			Moteur_Commande(200 ,2 );
			Delay10KTCYx(500);*/

		//	figure();
		//	fig = 0;
		/*	kv = 0;
			INTCONbits.PEIE=1;				
			INTCONbits.GIE=1;
			Delay10KTCYx(300);
			kv = KV;	*/		
	//	}
	}
	LED=0;					// Quand on est arriv,
	INTCONbits.PEIE=0;		// On arrete de faire tourner les moteur
	INTCONbits.GIE=0;
	Moteur_Commande(0 ,1 );	// On stop le robot
	Moteur_Commande(0 ,2 );
	while(1){
	};
}// Fin du prog
/****************************/
/*******************************************************/
/***************************************************************************************/
//----------------------------------------------------------------------------
// Vecteur d'interruptions de priorit basse (utilis pour le Timer)
#pragma code lowVector=0x18							// Vecteur d'interruption de priorit basse
void atInterruptLow(void)
{
	_asm goto Interrupt _endasm		// On saute  la fonction MyHighInterrupt
}
#pragma code 										// Retour  la zone de code
//----------------------------------------------------------------------------
// Routine de service de l'interruption basse priorite
#pragma interrupt Interrupt
void Interrupt(void){		
	static int distance=0;
	static int distance_prim=0;
	static int distance_prim_mem=0;
	static int distance_prim_temp=0;
	static int distance_temp=0;
	static char i=0;
	int VG,VD;
	static char fin=0;
	if(PIR2bits.TMR3IF)					// Vrifie qu'il s'agit bien de l'interruption du Timer 3
	{											

		WriteTimer3(T_TIMER3);			// Recharge le compteur du Timer 3		
		PIR2bits.TMR3IF=0;				// Rautotise l'interruption
		
		distance = int_I2C (); 			// On interroge les capteurs	
		distance_prim = int_I2C_AR ();
		if ( distance_prim == 0x55 ) distance_prim= distance_prim_mem;
		distance_prim_mem = distance_prim;
		if ( distance != 0x55 ){		// Si valeur cohrente...
////////////////////////////////////////////////////////////////
			if ( distance == 0xFF ) fig=1;	// Si c'est une figure...
			else if ( distance == 0xF0 ) prio = 1;
			else {						// Sinon on calcul les vitesse des moteurs

				/*if ( (distance>50) || (distance<-50) ) {
					distance*=3;
					distance_prim*=3;
				} */
				if ( (((distance > 0) && (distance_prim > 0)) || ((distance < 0) && (distance_prim < 0))) && (distance != distance_prim) ){
						if ( ((distance>=0) && ( distance<distance_prim)) || ((distance<=0) && ( distance>distance_prim)) ) {
							VD = calcul_V ( -( distance - distance_prim), kv );
							VG = calcul_V (  ( distance - distance_prim), kv );
						}
						else {
							VD = calcul_V ( -( distance + distance_prim), kv );
							VG = calcul_V (  ( distance + distance_prim), kv );
						}
				}
				else {
					VD = calcul_V ( -( distance ), kv );
					VG = calcul_V (  ( distance ), kv );
				}
				
		
				Moteur_Commande(-(VG) ,1 );	// Et on l'envoi
				Moteur_Commande(-(VD) ,2 );

	
				distance_temp	   = distance;
			}
////////////////////////////////////////////////////////////////
		}
	}
}
//----------------------------------------------------------------------------
void figure ( void ){
	char ett = 1;
	char i=0, j=0;
	char k=0, l=0;

	int V=0;
	
	long int codD=2;
	long int codG=2;

	Moteur_Commande(-V_FIG ,1 );
	Moteur_Commande( V_FIG+45 ,2 );

	while (ett) {

		if ( COD_D ) {
			if ( !i ) codD++;
			i=1;
		}
		if ( !COD_D ) {
			if ( i ) codD++;
			i=0;
		}

		if ( COD_G ) {
			if ( !j ) codG++;
			j=1;
		}
		if ( !COD_G ) {
			if ( j ) codG++;
			j=0;
		}

		if ( codD>(INC_FIG ) )  { Moteur_Commande(0 ,1 ); k=1; } // +1500
		if ( codG>(INC_FIG ) )  { Moteur_Commande(0 ,2 ); l=1; } // -200
		
		if ( (k==1) && (l==1) ) ett=0;
	}
}

char laby ( void ) {
	int distanceG = 0;
	int distanceD = 0;
	int distance=0;
	
	if ( I2C_Receive(AdCPT_M) ) return 0;

	distanceD = CAN_Convert(ADC_CH1)+CAN_Convert(ADC_CH3);
	distanceG = CAN_Convert(ADC_CH0)+CAN_Convert(ADC_CH4);
	distance = distanceG-distanceD;
	Moteur_Commande( - distance/7 - 150 ,1 );
	Moteur_Commande(   distance/7 - 150 ,2 );

	return 1;
}

/*
	while ( !JACK ){	LED=!LED; Delay10KTCYx(200); }
	while (  JACK ){	LED=!LED; Delay10KTCYx(50);	 }
	LED=1;
	while ( laby() );
	
	kv = KV;
	INTCONbits.PEIE=1;			
	INTCONbits.GIE=1;
	
	while ( !CLIC ){
		if ( figure ) {
			INTCONbits.PEIE=0;			
			INTCONbits.GIE=0;
			figure ();
			INTCONbits.PEIE=1;			
			INTCONbits.GIE=1;
		}
		
		if ( ((CAN_Convert(ADC_CH0)>300) || (CAN_Convert(ADC_CH4)>300) || (CAN_Convert(ADC_CH2)>300)) && prio ) {
			priorite ();
		}
	}

	LED=0;
	INTCONbits.PEIE=0;			
	INTCONbits.GIE=0;
	Moteur_Commande( 0, 1 );
	Moteur_Commande( 0, 2 );
	while (1);
*/

